<?php

namespace app\models;

use moonland\phpexcel\Excel;
use carono\exchange1c\interfaces\ProductInterface;
use Zenwalker\CommerceML\Model\Product as Product1c;

class Product extends Document implements ProductInterface
{
    const SCENARIO_IMPORT = 'import';

    public $file;
    public $group;

    public static function collectionName()
    {
        return 'product';
    }

    public function attributes()
    {
        return array_merge(
            parent::attributes(),
            ['name', 'catn', 'oemn', 'data', 'cost', 'time']
        );
    }

    public function rules()
    {
        return array_merge(
            parent::rules(),
            [
                [
                    ['name', 'catn'],
                    'required',
                    'message' => 'Заполните поля: {attribute}',
                    'except' => self::SCENARIO_IMPORT
                ],
                [
                    'file',
                    'required',
                    'message' => 'Заполните поля: {attribute}',
                    'on' => self::SCENARIO_IMPORT
                ],
                ['catn', 'integer', 'message' => '{attribute} должен быть числом'],
                // ['oemn', 'integer'], Нужна своя проверка на массив
                [
                    'file',
                    'file',
                    'skipOnEmpty' => false,
                    'extensions' => 'xlsx',
                    'checkExtensionByMimeType' => false,
                    'maxFiles' => 5,
                    'maxSize' => 1024 * 1024 * 30,
                    'wrongExtension' => 'Разрешены только документы в формате: ".xlsx"',
                    'message' => 'Проблема при чтении документа',
                    'on' =>  self::SCENARIO_IMPORT
                ]
            ]
        );
    }

    public function attributeLabels()
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'name' => 'Название (name)',
                'catn' => 'Каталожный номер (catn)',
                'oemn' => 'OEM номера (oemn)',
                'data' => 'Данные товара (data)',
                'cost' => 'Цены (cost)',
                'time' => 'Сроки доставки (time)',
                'file' => 'Документ',
                'group' => 'Группа',
            ]
        );
    }

    public function beforeSave($data)
    {
        if (parent::beforeSave($data)) {
            if ($this->isNewRecord) {
                // Надо избавиться от unset();
                unset($this->_key);
            }

            return true;
        }

        return false;
    }

    public function import()
    {
        // Инициализация массива данных
        $data = [];

        if ($this->validate()) {
            foreach ($this->file as $file) {
                // Перебор файлов

                // Сохранение на диск
                if (!file_exists('../assets/import/excel/')) {
                    mkdir('../assets/import/excel/', 0775, true);
                }
                $file->saveAs($path = '../assets/import/excel/' . $file->baseName . '.' . $file->extension);

                // Проверка файла пройдена

                $data[] = Excel::import($path, [
                    'setFirstRecordAsKeys' => true,
                    'setIndexSheetByName' => true,
                ]);
            }

            foreach ($data[0] as $doc) {
                // Перебор полученных документов

                // Сохранение в базе данных
                $product = new static($doc);

                if ($product->validate()) {
                    // Проверка пройдена

                    // Запись документа
                    $product->save();

                    // Запись группы
                    $group = static::class . 'Group';
                    (new $group())->writeMember($product, $this->group);
                } else {
                    // Проверка не пройдена
                    foreach ($product->errors as $attribute => $error) {
                        $this->addError($attribute, $error);
                    }
                }
            }

            return true;
        }

        return false;
    }

    /**
     * Установка реквизитов для продукта
     */
    // public function setRequisite1c(string $name, string $value): bool
    public function setRequisite1c($name, $value): bool
    {
        if (!$requisite = Requisite::readByName($name)) {
            // Реквизиты не найдены

            // Инициализация
            $requisite = new Requisite();

            $requisite->name = $name;
            $requisite->value = $value;

            // Запись
            return $requisite->save();
        }

        return false;
    }

    /**
     * Установка группы, где находится продукт
     */
    // public function setGroup1c(ProductGroup $group): bool
    public function setGroup1c($group): bool
    {
        // Чтение группы
        $group = SupplyGroup::readByOnecName($group->id)[0];

        // Запись ребра: ПОСТАВКА => ГРУППА ПОСТАВОК
        return static::writeEdgeBetweenGroup(static::collectionName() . '/' . $this->_key, $group->collectionName() . '/' . $group->_key);
    }

    /**
     * Запись всех параметров. Вызывается 1 раз при импорте
     */
    public static function createProperties1c($properties): void
    {
        // Это нам не нужно, кажется, надо будет тестить

        /**
         * @var \Zenwalker\CommerceML\Model\Property $property
         */
        // foreach ($properties as $property) {
        //     $propertyModel = Property::createByMl($property);
        //     foreach ($property->getAvailableValues() as $value) {
        //         if (!$propertyValue = PropertyValue::findOne(['onec_name' => $value->id])) {
        //             $propertyValue = new PropertyValue();
        //             $propertyValue->name = (string)$value->Значение;
        //             $propertyValue->property_id = $propertyModel->id;
        //             $propertyValue->onec_name = (string)$value->ИдЗначения;
        //             $propertyValue->save();
        //             unset($propertyValue);
        //         }
        //     }
        // }
    }

    /**
     * $property - Свойство товара (import.xml > Классификатор > Свойства > Свойство)
     * $property->value - Разыменованное значение (string) (import.xml > Классификатор > Свойства > Свойство > Значение)
     * $property->getValueModel() - Данные по значению, Ид значения, и т.д (import.xml > Классификатор > Свойства > Свойство > ВариантыЗначений > Справочник)
     *
     * @param MlProperty $property
     * @return void
     */
    public function setProperty1c($property): void
    {
        // Это тоже нам не нужно

        // $propertyModel = Property::findOne(['onec_name' => $property->id]);
        // $propertyValue = $property->getValueModel();
        // if ($propertyAccountingId = (string)$propertyValue->ИдЗначения) {
        //     $value = PropertyValue::findOne(['onec_name' => $propertyAccountingId]);
        //     $attributes = ['property_value_id' => $value->id];
        // } else {
        //     $attributes = ['value' => $propertyValue->value];
        // }
        // $this->addPivot($propertyModel, PvProductProperty::class, $attributes);
    }

    /**
     * В этой фукнции мы получаем абсолютный путь до картинки и название изрбражения (для alt аттрибута)
     *
     * @param string $path
     * @param string $caption
     * @return mixed
     */
    public function addImage1c($path, $caption): bool
    {
        // if (!$this->getImages()->andWhere(['md5' => md5_file($path)])->exists()) {
        //     $this->addPivot(FileUpload::startUpload($path)->process(), PvProductImage::class, ['caption' => $caption]);
        // }

        return false;
    }

    /**
     * В эту фукнцию отправляется xml данные предложения из файла
     */
    // public function getOffer1c(Supply $offer): Supply
    public function getOffer1c($offer): Supply
    {
        $supply = Supply::createByMl($offer);
        $supply->product_id = $this->id;
        if ($supply->getDirtyAttributes()) {
            $supply->save();
        }

        return $supply;
    }

    /**
     * @param MlOffer $offer
     */
    public static function createByMl($offer): Supply
    {
        if (!$model = static::readByOnecName($offer->id)) {
            // Нет записей в базе данных

            // Инициализация
            $model = new static;

            $model->name = (string) $offer->name;
            $model->onec_name = (string) $offer->id;
        }

        $model->remnant = (string) $offer->Количество;

        return $model;
    }

    /**
     * @param  $product
     * @return self
     */
    public static function createModel1c($product): static
    {
        if (!$model = static::findOne(['onec_name' => $product->id])) {
            $model = new static();
            $model->onec_name = $product->id;
        }
        $model->name = $product->name;
        $model->description = (string) $product->Описание;
        $model->article = (string) $product->Артикул;

        $model->save();

        return $model;
    }

    public function setRaw1cData($cml, $object)
    {
    }

    /**
     * Название поля в котором хранится ID из 1C
     */
    public static function getIdFieldName1c(): string
    {
        return 'onec_name';
    }

    private static function writeEdgeBetweenGroup(string $from, string $to): bool
    {
        // Инициализация
        $edge = new SupplyEdgeSupplyGroup();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    private static function writeEdgeBetweenRequisite(string $from, string $to): bool
    {
        // Инициализация
        $edge = new SupplyEdgeRequisite();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    public static function readById(string $_key): ?Product
    {
        return Product::findOne(['_key' => $_key]);
    }

    public function getGroup1c(): ProductGroup
    {
        return $this->group;
    }

    public static function readByOnecName(string $name): ?Product
    {
        return static::findOne([static::getIdFieldName1c() => $name]);
    }
}
