<?php

namespace app\models;

use Yii;
use app\models\Account;
use app\models\Product;
use app\models\SupplyEdgeProduct;
use carono\exchange1c\interfaces\OfferInterface;

class Supply extends Product implements OfferInterface
{
    public static function collectionName()
    {
        return 'supply';
    }

    public function afterSave($data, $vars)
    {
        if (is_null($product = self::readByCatn($this->catn))) {
            // Товар не найден

            // Инициализация
            $product = (new Product(array_intersect_key($this->getAttributes(), (new Product)->getAttributes())));

            // Запись
            if (!$product->save()) {
                return false;
            }
        }

        // Запись рёбер: АККАУНТ => ПОСТАВКА => ТОВАР, и проверка на то, что оба созданы
        static::writeEdgeBetweenAccount(Account::collectionName() . '/' . Yii::$app->user->identity->_key, static::collectionName() . '/' . $this->_key);
        static::writeEdgeBetweenProduct(static::collectionName() . '/' . $this->_key, Product::collectionName() . '/' . $product->_key);
    }

    /**
     * В этом методе необходимо создать все типы цен, фукнция вызывается один раз
     */
    public static function createPriceTypes1c($types): void
    {
        foreach ($types as $type) {
            // PriceType::createByMl($type);
        }
    }

    /**
     * offers.xml > ПакетПредложений > Предложения > Предложение > Цены
     *
     * Цена товара,
     * К $price можно обратиться как к массиву, чтобы получить список цен (Цены > Цена)
     * $price->type - тип цены (offers.xml > ПакетПредложений > ТипыЦен > ТипЦены)
     *
     * @param \Zenwalker\CommerceML\Model\Price $price
     */
    public function setPrice1c($price): void
    {
        // $priceType = PriceType::findOne(['accounting_id' => $price->getType()->id]);
        // $priceModel = Price::createByMl($price, $this, $priceType);
        // $this->addPivot($priceModel, PvOfferPrice::class);
    }

    /**
     * offers.xml > ПакетПредложений > Предложения > Предложение > ХарактеристикиТовара > ХарактеристикаТовара
     *
     * Характеристики товара
     * $name - Наименование
     * $value - Значение
     *
     * @param \Zenwalker\CommerceML\Model\Simple $specification
     * @return void
     */
    public function setSpecification1c($specification)
    {
        // $specificationModel = Specification::createByMl($specification);
        // $this->addPivot($specificationModel, PvOfferSpecification::class, ['value' => (string)$specification->Значение]);
    }

    public function getExportFields1c($context = null)
    {
        return [
            'Ид' => 'id',
            'Наименование' => 'login',
            'ПолноеНаименование' => 'full_name',
            'Фамилия' => 'surname',
            'Имя' => 'name',
            'Контакты' => [
                [
                    '@name' => 'Контакт',
                    'Тип' => 'Почта',
                    'Значение' => $this->email,
                ],
                [
                    '@name' => 'Контакт',
                    'Тип' => 'ТелефонРабочий',
                    'Значение' => $this->phone,
                ],
            ],
        ];
    }

    public static function writeEdgeBetweenAccount(string $from, string $to): bool
    {
        // Инициализация
        $edge = new AccountEdgeSupply();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    private static function writeEdgeBetweenProduct(string $from, string $to): bool
    {
        // Инициализация
        $edge = new SupplyEdgeProduct();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    private static function writeEdgeBetweenGroup(string $from, string $to): bool
    {
        // Инициализация
        $edge = new SupplyEdgeSupplyGroup();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    private static function writeEdgeBetweenRequisite(string $from, string $to): bool
    {
        // Инициализация
        $edge = new SupplyEdgeRequisite();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    public static function readByCatn(string $catn): ?Product
    {
        return Product::findOne(['catn' => $catn]);
    }
}
